@@ -95,6 +95,8 @@ class JwtParam(BaseModel):
     email: EmailStr
     first_name: str
     role: RoleEnum
+    org_id: str
+    user_id: str
 
 
 class MagicLinkDto(BaseModel):
@@ -129,6 +131,44 @@ class WhoAmI(BaseModel):
     email: EmailStr
     firstName: str
     role: RoleEnum
+    orgId: str
+    userId: str
+
+
+class UserEmailAndId(BaseModel):
+    emailAddress: EmailStr
+    userId: str
+
+
+class EmailDomainAndOrgId(BaseModel):
+    emailDomain: str
+    orgId: str
+
+
+class CategorySummary(BaseModel):
+    categoryId: str
+    categoryName: str
+    description: str
+    hasUnfilled: Optional[bool]
+
+
+class CategorySummaries(BaseModel):
+    __root__: List[CategorySummary]
+
+
+class EvaluationSummary(BaseModel):
+    evaluationId: str
+    evaluationName: str
+    matId: str
+    status: str
+    createDate: str
+    dueDate: str
+    fillDate: Optional[str]
+    targetGroup: str
+
+
+class EvaluationSummaries(BaseModel):
+    __root__: List[Optional[EvaluationSummary]]
 
 
 #
