@@ -1,6 +1,7 @@
 import logging
 import os
 import re
+from typing import Union
 
 import tomli
 from fastapi import APIRouter
@@ -16,7 +17,7 @@ router = APIRouter(
 )
 
 
-def get_current_version():
+def get_current_version() -> Union[str, None]:
     current_version = None
     try:
         current_file_dir = os.path.dirname(os.path.realpath("__file__"))
