@@ -18,8 +18,10 @@ from fastapi import APIRouter, Request
 from fastapi.responses import JSONResponse
 from jinja2 import Template
 from mypy_boto3_secretsmanager import SecretsManagerClient
+from mypy_boto3_s3.type_defs import PutObjectOutputTypeDef
 from pydantic import EmailStr
 
+
 from ..aws_operations import (
     get_secret_binary,
     s3_get_object_bytes,
@@ -39,7 +41,14 @@ from ..custom_types import (
     MagicLinkInternal,
     RoleEnum,
     WhoAmI,
+    UserEmailAndId,
+    EmailDomainAndOrgId,
+    CategorySummary,
+    CategorySummaries,
+    EvaluationSummary,
+    EvaluationSummaries,
 )
+from ..eid import get_org_eid, get_user_eid
 from ..http_return import (
     http_200_json,
     http_400_json,
@@ -148,6 +157,8 @@ class Auth:
                     "iss": issuer,
                     "nbf": now,
                     "role": jwt_param.role,
+                    "org_id": jwt_param.org_id,
+                    "user_id": jwt_param.user_id,
                 },
                 signing_key.to_pem(),
                 algorithm=algorithm,
@@ -537,6 +548,271 @@ class Auth:
                 reasons=[f"{ex}"],
             )
 
+    def get_organization_id_for_user(
+        self, email_address: EmailStr
+    ) -> Union[str, CustomError]:
+        try:
+            error_msg = f"Could not get organization id for: {email_address}"
+
+            email_domain = email_address.split("@")[-1]
+            s3_response = s3_get_object_bytes(
+                s3_client=Auth.clients.s3_client,
+                s3_bucket=self.config.s3_bucket,
+                s3_key=f"api/index/email-domain/{email_domain}.json",
+            )
+
+            if isinstance(s3_response, HttpError) and s3_response.status_code != 404:
+                LOG.error(s3_response)
+                return CustomError(
+                    message=s3_response.error.message,
+                    reasons=s3_response.error.reasons,
+                )
+
+            if isinstance(s3_response, HttpError) and s3_response.status_code == 404:
+                org_id = get_org_eid("examplecorp.com")
+
+            else:
+                domain_and_org_id = EmailDomainAndOrgId.parse_raw(s3_response)
+                org_id = domain_and_org_id.orgId
+
+            return org_id
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def put_user_email_and_id(
+        self, email_address: EmailStr, user_id: str, org_id: str
+    ) -> Union[PutObjectOutputTypeDef, CustomError]:
+        try:
+            error_msg = f"Could not put user email and user id: {user_id}"
+            user_email_and_id = UserEmailAndId(
+                emailAddress=email_address, userId=user_id
+            )
+            s3_response = s3_put_object_bytes(
+                Auth.clients.s3_client,
+                self.config.s3_bucket,
+                f"api/index/email/{org_id}/{user_id}.json",
+                user_email_and_id.json().encode("utf-8"),
+            )
+
+            if isinstance(s3_response, HttpError):
+                LOG.error(s3_response)
+                return CustomError(
+                    message=s3_response.error.message,
+                    reasons=s3_response.error.reasons,
+                )
+            else:
+                return s3_response
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def put_category_summaries(
+        self, user_id: str
+    ) -> Union[PutObjectOutputTypeDef, CustomError]:
+        try:
+            error_msg = f"Could not put category summary: {user_id}"
+            cats = [
+                CategorySummary(
+                    categoryId="cat-xbyckrvqbjbpq2aoboa7o4zb7y",
+                    categoryName="360º Review",
+                    description="A 360-degree feedback is ...",
+                    hasUnfilled=False,
+                ),
+                CategorySummary(
+                    categoryId="cat-pj4ofwyimnapusd3toqoetb2ou",
+                    categoryName="Employee Satisfaction",
+                    description="How satisifed the employees are",
+                    hasUnfilled=True,
+                ),
+                CategorySummary(
+                    categoryId="cat-h54wcy7l7fdrrxq425mcmvoaci",
+                    categoryName="Other",
+                    description="Miscellaneous",
+                    hasUnfilled=False,
+                ),
+            ]
+            category_summaries = CategorySummaries.parse_obj(cats)
+            s3_response = s3_put_object_bytes(
+                Auth.clients.s3_client,
+                self.config.s3_bucket,
+                f"api/index/evals/{user_id}/category-summaries.json",
+                category_summaries.json().encode("utf-8"),
+            )
+
+            if isinstance(s3_response, HttpError):
+                LOG.error(s3_response)
+                return CustomError(
+                    message=s3_response.error.message,
+                    reasons=s3_response.error.reasons,
+                )
+            else:
+                return s3_response
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def put_favorite_drinks_eval(
+        self, user_id: str
+    ) -> Union[PutObjectOutputTypeDef, CustomError]:
+        try:
+            error_msg = f"Could not put evaluation starter pack: {user_id}"
+            favorite_drinks_eval = [
+                EvaluationSummary(
+                    evaluationId="eva-pze4xhqb5rbmedhqrmgd2l7i6m",
+                    evaluationName="Favorite Drinks",
+                    matId="mat-7wby5o6exxce3md5yczyv26y5m",
+                    status="unfilled",
+                    createDate="2023-01-11",
+                    dueDate="2023-09-30",
+                    targetGroup="All",
+                )
+            ]
+            evaluation_summaries = EvaluationSummaries.parse_obj(favorite_drinks_eval)
+            s3_response = s3_put_object_bytes(
+                Auth.clients.s3_client,
+                self.config.s3_bucket,
+                f"api/index/evals/{user_id}/cat-pj4ofwyimnapusd3toqoetb2ou/unfilled.json",
+                evaluation_summaries.json().encode("utf-8"),
+            )
+
+            if isinstance(s3_response, HttpError):
+                LOG.error(s3_response)
+                return CustomError(
+                    message=s3_response.error.message,
+                    reasons=s3_response.error.reasons,
+                )
+            else:
+                return s3_response
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def put_empty_eval_summaries(self, user_id: str) -> Union[None, CustomError]:
+        try:
+            error_msg = f"Could not put empty evaluation summaries: {user_id}"
+            empty_file = EvaluationSummaries(__root__=[])
+
+            keys = [
+                f"api/index/evals/{user_id}/cat-pj4ofwyimnapusd3toqoetb2ou/filled.json",
+                f"api/index/evals/{user_id}/cat-xbyckrvqbjbpq2aoboa7o4zb7y/filled.json",
+                f"api/index/evals/{user_id}/cat-xbyckrvqbjbpq2aoboa7o4zb7y/unfilled.json",
+                f"api/index/evals/{user_id}/cat-h54wcy7l7fdrrxq425mcmvoaci/filled.json",
+                f"api/index/evals/{user_id}/cat-h54wcy7l7fdrrxq425mcmvoaci/unfilled.json",
+            ]
+            bu = self.config.s3_bucket
+            cl = Auth.clients.s3_client
+            bo = empty_file.json().encode("utf-8")
+
+            s3_responses = list(
+                map(lambda key: s3_put_object_bytes(cl, bu, key, bo), keys)
+            )
+            error_maybe = list(filter(lambda x: isinstance(x, HttpError), s3_responses))
+            if len(error_maybe) > 0:
+                LOG.error(error_maybe)
+                return CustomError(
+                    message=error_msg,
+                    reasons=error_maybe,
+                )
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def create_user(self, email_address: EmailStr, user_id: str, org_id: str):
+        try:
+            error_msg = f"Could not create user: {user_id}"
+
+            s3_responses = []
+
+            user_email_and_id_response = self.put_user_email_and_id(
+                email_address, user_id, org_id
+            )
+            s3_responses.append(user_email_and_id_response)
+
+            category_summaries_response = self.put_category_summaries(user_id)
+            s3_responses.append(category_summaries_response)
+
+            favorite_drinks_eval_repsponse = self.put_favorite_drinks_eval(user_id)
+            s3_responses.append(favorite_drinks_eval_repsponse)
+
+            put_empty_evals_response = self.put_empty_eval_summaries(user_id)
+            s3_responses.append(put_empty_evals_response)
+
+            error_maybe = list(
+                filter(lambda x: isinstance(x, CustomError), s3_responses)
+            )
+            if len(error_maybe) > 0:
+                LOG.error(error_maybe)
+                return CustomError(
+                    message="Error while talking to aws s3",
+                    reasons=error_maybe,
+                )
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
+    def get_user_id_for_user(
+        self, org_id: str, email_address: EmailStr
+    ) -> Union[str, CustomError]:
+        try:
+            error_msg = f"Could not get user id for: {email_address}"
+
+            user_id = get_user_eid(email_address)
+            s3_response = s3_get_object_bytes(
+                s3_client=Auth.clients.s3_client,
+                s3_bucket=self.config.s3_bucket,
+                s3_key=f"api/index/email/{org_id}/{user_id}.json",
+            )
+
+            if isinstance(s3_response, HttpError) and s3_response.status_code != 404:
+                LOG.error(s3_response)
+                return CustomError(
+                    message=s3_response.error.message,
+                    reasons=s3_response.error.reasons,
+                )
+
+            if isinstance(s3_response, HttpError) and s3_response.status_code == 404:
+                user_creation_maybe = self.create_user(email_address, user_id, org_id)
+                if isinstance(user_creation_maybe, CustomError):
+                    LOG.error(user_creation_maybe)
+                    return CustomError(
+                        message=user_creation_maybe.message,
+                        reasons=user_creation_maybe.reasons,
+                    )
+
+            return user_id
+
+        except Exception as ex:
+            LOG.error(error_msg)
+            return CustomError(
+                message=error_msg,
+                reasons=[f"{ex}"],
+            )
+
     # @router.post("/login", response_model=HttpSuccess)
     async def login(self, login_hash: LoginHash) -> JSONResponse:
         try:
@@ -561,10 +837,27 @@ class Auth:
                     message=mli_maybe.error.message, reasons=mli_maybe.error.reasons
                 )
 
+            email_address = mli_maybe.magic_link.email
+            org_id_maybe = self.get_organization_id_for_user(email_address)
+            if isinstance(org_id_maybe, CustomError):
+                LOG.error(f"{org_id_maybe}")
+                return http_500_json(
+                    message=org_id_maybe.message, reasons=org_id_maybe.reasons
+                )
+
+            user_id_maybe = self.get_user_id_for_user(org_id_maybe, email_address)
+            if isinstance(user_id_maybe, CustomError):
+                LOG.error(f"{user_id_maybe}")
+                return http_500_json(
+                    message=user_id_maybe.message, reasons=user_id_maybe.reasons
+                )
+
             jwt_param = JwtParam(
                 email=mli_maybe.magic_link.email,
                 first_name=mli_maybe.magic_link.first_name,
                 role=RoleEnum.user,
+                org_id=org_id_maybe,
+                user_id=user_id_maybe,
             )
 
             jwt_token_maybe = Auth.create_jwt(
@@ -623,7 +916,11 @@ class Auth:
 
             jwt_token_maybe = Auth.create_jwt(
                 jwt_param=JwtParam(
-                    email="log@out.com", first_name="Logout", role=RoleEnum.user
+                    email="log@out.com",
+                    first_name="Logout",
+                    role=RoleEnum.user,
+                    org_id="",
+                    user_id="",
                 ),
                 signing_key=signing_key_maybe,
                 audience=self.config.jwt_audience,
@@ -660,7 +957,7 @@ class Auth:
                 self.config.jwt_name,
                 signing_key_maybe,
                 self.config.jwt_audience,
-                ["email", "first_name", "role"],
+                ["email", "first_name", "role", "org_id", "user_id"],
             )
             if isinstance(token_fields_maybe, CustomError):
                 return http_403_json(
@@ -681,10 +978,16 @@ class Auth:
                     reasons=reasons,
                 )
 
-            email, first_name, role = token_fields_maybe
+            email, first_name, role, org_id, user_id = token_fields_maybe
 
             return http_200_json(
-                WhoAmI(email=EmailStr(email), firstName=first_name, role=role)
+                WhoAmI(
+                    email=EmailStr(email),
+                    firstName=first_name,
+                    role=role,
+                    orgId=org_id,
+                    userId=user_id,
+                )
             )
         except Exception as ex:
             message, reasons = "WhoAmI Error", [f"{ex}"]
